r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Media
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page


class MediaRecordingInstance(InstanceResource):
    class Format(object):
        MP4 = "mp4"
        WEBM = "webm"

    class Order(object):
        ASC = "asc"
        DESC = "desc"

    class Status(object):
        PROCESSING = "processing"
        COMPLETED = "completed"
        DELETED = "deleted"
        FAILED = "failed"

    def __init__(self, version, payload, sid: Optional[str] = None):
        """
        Initialize the MediaRecordingInstance

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """
        super().__init__(version)

        self._properties = {
            "account_sid": payload.get("account_sid"),
            "date_created": deserialize.iso8601_datetime(payload.get("date_created")),
            "date_updated": deserialize.iso8601_datetime(payload.get("date_updated")),
            "duration": deserialize.integer(payload.get("duration")),
            "format": payload.get("format"),
            "links": payload.get("links"),
            "processor_sid": payload.get("processor_sid"),
            "resolution": payload.get("resolution"),
            "source_sid": payload.get("source_sid"),
            "sid": payload.get("sid"),
            "media_size": payload.get("media_size"),
            "status": payload.get("status"),
            "status_callback": payload.get("status_callback"),
            "status_callback_method": payload.get("status_callback_method"),
            "url": payload.get("url"),
        }

        self._solution = {
            "sid": sid or self._properties["sid"],
        }
        self._context: Optional[MediaRecordingContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: MediaRecordingContext for this MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingContext
        """
        if self._context is None:
            self._context = MediaRecordingContext(
                self._version,
                sid=self._solution["sid"],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the MediaRecording resource.
        :rtype: str
        """
        return self._properties["account_sid"]

    @property
    def date_created(self):
        """
        :returns: The date and time in GMT when the resource was created specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_created"]

    @property
    def date_updated(self):
        """
        :returns: The date and time in GMT when the resource was last updated specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_updated"]

    @property
    def duration(self):
        """
        :returns: The duration of the MediaRecording in seconds.
        :rtype: int
        """
        return self._properties["duration"]

    @property
    def format(self):
        """
        :returns:
        :rtype: MediaRecordingInstance.Format
        """
        return self._properties["format"]

    @property
    def links(self):
        """
        :returns: The URLs of related resources.
        :rtype: dict
        """
        return self._properties["links"]

    @property
    def processor_sid(self):
        """
        :returns: The SID of the MediaProcessor resource which produced the MediaRecording.
        :rtype: str
        """
        return self._properties["processor_sid"]

    @property
    def resolution(self):
        """
        :returns: The dimensions of the video image in pixels expressed as columns (width) and rows (height).
        :rtype: str
        """
        return self._properties["resolution"]

    @property
    def source_sid(self):
        """
        :returns: The SID of the resource that generated the original media track(s) of the MediaRecording.
        :rtype: str
        """
        return self._properties["source_sid"]

    @property
    def sid(self):
        """
        :returns: The unique string generated to identify the MediaRecording resource.
        :rtype: str
        """
        return self._properties["sid"]

    @property
    def media_size(self):
        """
        :returns: The size of the recording media in bytes.
        :rtype: int
        """
        return self._properties["media_size"]

    @property
    def status(self):
        """
        :returns:
        :rtype: MediaRecordingInstance.Status
        """
        return self._properties["status"]

    @property
    def status_callback(self):
        """
        :returns: The URL to which Twilio will send asynchronous webhook requests for every MediaRecording event. See [Status Callbacks](/docs/live/status-callbacks) for more details.
        :rtype: str
        """
        return self._properties["status_callback"]

    @property
    def status_callback_method(self):
        """
        :returns: The HTTP method Twilio should use to call the `status_callback` URL. Can be `POST` or `GET` and the default is `POST`.
        :rtype: str
        """
        return self._properties["status_callback_method"]

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource.
        :rtype: str
        """
        return self._properties["url"]

    def delete(self):
        """
        Deletes the MediaRecordingInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    async def delete_async(self):
        """
        Asynchronous coroutine that deletes the MediaRecordingInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return await self._proxy.delete_async()

    def fetch(self):
        """
        Fetch the MediaRecordingInstance


        :returns: The fetched MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the MediaRecordingInstance


        :returns: The fetched MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """
        return await self._proxy.fetch_async()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Media.V1.MediaRecordingInstance {}>".format(context)


class MediaRecordingContext(InstanceContext):
    def __init__(self, version: Version, sid: str):
        """
        Initialize the MediaRecordingContext

        :param Version version: Version that contains the resource
        :param sid: The SID of the MediaRecording resource to fetch.

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingContext
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "sid": sid,
        }
        self._uri = "/MediaRecordings/{sid}".format(**self._solution)

    def delete(self):
        """
        Deletes the MediaRecordingInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(
            method="DELETE",
            uri=self._uri,
        )

    async def delete_async(self):
        """
        Asynchronous coroutine that deletes the MediaRecordingInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return await self._version.delete_async(
            method="DELETE",
            uri=self._uri,
        )

    def fetch(self):
        """
        Fetch the MediaRecordingInstance


        :returns: The fetched MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return MediaRecordingInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the MediaRecordingInstance


        :returns: The fetched MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return MediaRecordingInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Media.V1.MediaRecordingContext {}>".format(context)


class MediaRecordingPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of MediaRecordingInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """
        return MediaRecordingInstance(self._version, payload)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Media.V1.MediaRecordingPage>"


class MediaRecordingList(ListResource):
    def __init__(self, version: Version):
        """
        Initialize the MediaRecordingList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingList
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingList
        """
        super().__init__(version)

        self._uri = "/MediaRecordings"

    def stream(
        self,
        order=values.unset,
        status=values.unset,
        processor_sid=values.unset,
        source_sid=values.unset,
        limit=None,
        page_size=None,
    ):
        """
        Streams MediaRecordingInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param MediaRecordingInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaRecordingInstance.Status status: Status to filter by, with possible values `processing`, `completed`, `deleted`, or `failed`.
        :param str processor_sid: SID of a MediaProcessor to filter by.
        :param str source_sid: SID of a MediaRecording source to filter by.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_recording.MediaRecordingInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(
            order=order,
            status=status,
            processor_sid=processor_sid,
            source_sid=source_sid,
            page_size=limits["page_size"],
        )

        return self._version.stream(page, limits["limit"])

    async def stream_async(
        self,
        order=values.unset,
        status=values.unset,
        processor_sid=values.unset,
        source_sid=values.unset,
        limit=None,
        page_size=None,
    ):
        """
        Asynchronously streams MediaRecordingInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param MediaRecordingInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaRecordingInstance.Status status: Status to filter by, with possible values `processing`, `completed`, `deleted`, or `failed`.
        :param str processor_sid: SID of a MediaProcessor to filter by.
        :param str source_sid: SID of a MediaRecording source to filter by.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_recording.MediaRecordingInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(
            order=order,
            status=status,
            processor_sid=processor_sid,
            source_sid=source_sid,
            page_size=limits["page_size"],
        )

        return await self._version.stream_async(page, limits["limit"])

    def list(
        self,
        order=values.unset,
        status=values.unset,
        processor_sid=values.unset,
        source_sid=values.unset,
        limit=None,
        page_size=None,
    ):
        """
        Lists MediaRecordingInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param MediaRecordingInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaRecordingInstance.Status status: Status to filter by, with possible values `processing`, `completed`, `deleted`, or `failed`.
        :param str processor_sid: SID of a MediaProcessor to filter by.
        :param str source_sid: SID of a MediaRecording source to filter by.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_recording.MediaRecordingInstance]
        """
        return list(
            self.stream(
                order=order,
                status=status,
                processor_sid=processor_sid,
                source_sid=source_sid,
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(
        self,
        order=values.unset,
        status=values.unset,
        processor_sid=values.unset,
        source_sid=values.unset,
        limit=None,
        page_size=None,
    ):
        """
        Asynchronously lists MediaRecordingInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param MediaRecordingInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaRecordingInstance.Status status: Status to filter by, with possible values `processing`, `completed`, `deleted`, or `failed`.
        :param str processor_sid: SID of a MediaProcessor to filter by.
        :param str source_sid: SID of a MediaRecording source to filter by.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_recording.MediaRecordingInstance]
        """
        return list(
            await self.stream_async(
                order=order,
                status=status,
                processor_sid=processor_sid,
                source_sid=source_sid,
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self,
        order=values.unset,
        status=values.unset,
        processor_sid=values.unset,
        source_sid=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Retrieve a single page of MediaRecordingInstance records from the API.
        Request is executed immediately

        :param MediaRecordingInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaRecordingInstance.Status status: Status to filter by, with possible values `processing`, `completed`, `deleted`, or `failed`.
        :param str processor_sid: SID of a MediaProcessor to filter by.
        :param str source_sid: SID of a MediaRecording source to filter by.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingPage
        """
        data = values.of(
            {
                "Order": order,
                "Status": status,
                "ProcessorSid": processor_sid,
                "SourceSid": source_sid,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return MediaRecordingPage(self._version, response)

    async def page_async(
        self,
        order=values.unset,
        status=values.unset,
        processor_sid=values.unset,
        source_sid=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Asynchronously retrieve a single page of MediaRecordingInstance records from the API.
        Request is executed immediately

        :param MediaRecordingInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaRecordingInstance.Status status: Status to filter by, with possible values `processing`, `completed`, `deleted`, or `failed`.
        :param str processor_sid: SID of a MediaProcessor to filter by.
        :param str source_sid: SID of a MediaRecording source to filter by.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingPage
        """
        data = values.of(
            {
                "Order": order,
                "Status": status,
                "ProcessorSid": processor_sid,
                "SourceSid": source_sid,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return MediaRecordingPage(self._version, response)

    def get_page(self, target_url):
        """
        Retrieve a specific page of MediaRecordingInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return MediaRecordingPage(self._version, response)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of MediaRecordingInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return MediaRecordingPage(self._version, response)

    def get(self, sid):
        """
        Constructs a MediaRecordingContext

        :param sid: The SID of the MediaRecording resource to fetch.

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingContext
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingContext
        """
        return MediaRecordingContext(self._version, sid=sid)

    def __call__(self, sid):
        """
        Constructs a MediaRecordingContext

        :param sid: The SID of the MediaRecording resource to fetch.

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingContext
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingContext
        """
        return MediaRecordingContext(self._version, sid=sid)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Media.V1.MediaRecordingList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Media
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page
from twilio.rest.media.v1.player_streamer.playback_grant import PlaybackGrantList


class PlayerStreamerInstance(InstanceResource):
    class EndedReason(object):
        ENDED_VIA_API = "ended-via-api"
        MAX_DURATION_EXCEEDED = "max-duration-exceeded"
        STREAM_DISCONNECTED_BY_SOURCE = "stream-disconnected-by-source"
        UNEXPECTED_FAILURE = "unexpected-failure"

    class Order(object):
        ASC = "asc"
        DESC = "desc"

    class Status(object):
        CREATED = "created"
        STARTED = "started"
        ENDED = "ended"
        FAILED = "failed"

    def __init__(self, version, payload, sid: Optional[str] = None):
        """
        Initialize the PlayerStreamerInstance

        :returns: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        super().__init__(version)

        self._properties = {
            "account_sid": payload.get("account_sid"),
            "date_created": deserialize.iso8601_datetime(payload.get("date_created")),
            "date_updated": deserialize.iso8601_datetime(payload.get("date_updated")),
            "video": payload.get("video"),
            "links": payload.get("links"),
            "sid": payload.get("sid"),
            "status": payload.get("status"),
            "url": payload.get("url"),
            "status_callback": payload.get("status_callback"),
            "status_callback_method": payload.get("status_callback_method"),
            "ended_reason": payload.get("ended_reason"),
            "max_duration": deserialize.integer(payload.get("max_duration")),
        }

        self._solution = {
            "sid": sid or self._properties["sid"],
        }
        self._context: Optional[PlayerStreamerContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: PlayerStreamerContext for this PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerContext
        """
        if self._context is None:
            self._context = PlayerStreamerContext(
                self._version,
                sid=self._solution["sid"],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the PlayerStreamer resource.
        :rtype: str
        """
        return self._properties["account_sid"]

    @property
    def date_created(self):
        """
        :returns: The date and time in GMT when the resource was created specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_created"]

    @property
    def date_updated(self):
        """
        :returns: The date and time in GMT when the resource was last updated specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_updated"]

    @property
    def video(self):
        """
        :returns: Specifies whether the PlayerStreamer is configured to stream video. Defaults to `true`.
        :rtype: bool
        """
        return self._properties["video"]

    @property
    def links(self):
        """
        :returns: The URLs of related resources.
        :rtype: dict
        """
        return self._properties["links"]

    @property
    def sid(self):
        """
        :returns: The unique string generated to identify the PlayerStreamer resource.
        :rtype: str
        """
        return self._properties["sid"]

    @property
    def status(self):
        """
        :returns:
        :rtype: PlayerStreamerInstance.Status
        """
        return self._properties["status"]

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource.
        :rtype: str
        """
        return self._properties["url"]

    @property
    def status_callback(self):
        """
        :returns: The URL to which Twilio will send asynchronous webhook requests for every PlayerStreamer event. See [Status Callbacks](/docs/live/status-callbacks) for more details.
        :rtype: str
        """
        return self._properties["status_callback"]

    @property
    def status_callback_method(self):
        """
        :returns: The HTTP method Twilio should use to call the `status_callback` URL. Can be `POST` or `GET` and the default is `POST`.
        :rtype: str
        """
        return self._properties["status_callback_method"]

    @property
    def ended_reason(self):
        """
        :returns:
        :rtype: PlayerStreamerInstance.EndedReason
        """
        return self._properties["ended_reason"]

    @property
    def max_duration(self):
        """
        :returns: The maximum time, in seconds, that the PlayerStreamer is active (`created` or `started`) before automatically ends. The default value is 300 seconds, and the maximum value is 90000 seconds. Once this maximum duration is reached, Twilio will end the PlayerStreamer, regardless of whether media is still streaming.
        :rtype: int
        """
        return self._properties["max_duration"]

    def fetch(self):
        """
        Fetch the PlayerStreamerInstance


        :returns: The fetched PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the PlayerStreamerInstance


        :returns: The fetched PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        return await self._proxy.fetch_async()

    def update(self, status):
        """
        Update the PlayerStreamerInstance

        :param PlayerStreamerInstance.UpdateStatus status:

        :returns: The updated PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        return self._proxy.update(
            status=status,
        )

    async def update_async(self, status):
        """
        Asynchronous coroutine to update the PlayerStreamerInstance

        :param PlayerStreamerInstance.UpdateStatus status:

        :returns: The updated PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        return await self._proxy.update_async(
            status=status,
        )

    @property
    def playback_grant(self):
        """
        Access the playback_grant

        :returns: twilio.rest.media.v1.player_streamer.PlaybackGrantList
        :rtype: twilio.rest.media.v1.player_streamer.PlaybackGrantList
        """
        return self._proxy.playback_grant

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Media.V1.PlayerStreamerInstance {}>".format(context)


class PlayerStreamerContext(InstanceContext):
    def __init__(self, version: Version, sid: str):
        """
        Initialize the PlayerStreamerContext

        :param Version version: Version that contains the resource
        :param sid: The SID of the PlayerStreamer resource to update.

        :returns: twilio.rest.media.v1.player_streamer.PlayerStreamerContext
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "sid": sid,
        }
        self._uri = "/PlayerStreamers/{sid}".format(**self._solution)

        self._playback_grant: Optional[PlaybackGrantList] = None

    def fetch(self):
        """
        Fetch the PlayerStreamerInstance


        :returns: The fetched PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return PlayerStreamerInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the PlayerStreamerInstance


        :returns: The fetched PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return PlayerStreamerInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    def update(self, status):
        """
        Update the PlayerStreamerInstance

        :param PlayerStreamerInstance.UpdateStatus status:

        :returns: The updated PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        data = values.of(
            {
                "Status": status,
            }
        )

        payload = self._version.update(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return PlayerStreamerInstance(self._version, payload, sid=self._solution["sid"])

    async def update_async(self, status):
        """
        Asynchronous coroutine to update the PlayerStreamerInstance

        :param PlayerStreamerInstance.UpdateStatus status:

        :returns: The updated PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        data = values.of(
            {
                "Status": status,
            }
        )

        payload = await self._version.update_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return PlayerStreamerInstance(self._version, payload, sid=self._solution["sid"])

    @property
    def playback_grant(self):
        """
        Access the playback_grant

        :returns: twilio.rest.media.v1.player_streamer.PlaybackGrantList
        :rtype: twilio.rest.media.v1.player_streamer.PlaybackGrantList
        """
        if self._playback_grant is None:
            self._playback_grant = PlaybackGrantList(
                self._version,
                self._solution["sid"],
            )
        return self._playback_grant

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Media.V1.PlayerStreamerContext {}>".format(context)


class PlayerStreamerPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of PlayerStreamerInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        return PlayerStreamerInstance(self._version, payload)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Media.V1.PlayerStreamerPage>"


class PlayerStreamerList(ListResource):
    def __init__(self, version: Version):
        """
        Initialize the PlayerStreamerList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.media.v1.player_streamer.PlayerStreamerList
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerList
        """
        super().__init__(version)

        self._uri = "/PlayerStreamers"

    def create(
        self,
        video=values.unset,
        status_callback=values.unset,
        status_callback_method=values.unset,
        max_duration=values.unset,
    ):
        """
        Create the PlayerStreamerInstance

        :param bool video: Specifies whether the PlayerStreamer is configured to stream video. Defaults to `true`.
        :param str status_callback: The URL to which Twilio will send asynchronous webhook requests for every PlayerStreamer event. See [Status Callbacks](/docs/live/status-callbacks) for more details.
        :param str status_callback_method: The HTTP method Twilio should use to call the `status_callback` URL. Can be `POST` or `GET` and the default is `POST`.
        :param int max_duration: The maximum time, in seconds, that the PlayerStreamer is active (`created` or `started`) before automatically ends. The default value is 300 seconds, and the maximum value is 90000 seconds. Once this maximum duration is reached, Twilio will end the PlayerStreamer, regardless of whether media is still streaming.

        :returns: The created PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        data = values.of(
            {
                "Video": video,
                "StatusCallback": status_callback,
                "StatusCallbackMethod": status_callback_method,
                "MaxDuration": max_duration,
            }
        )

        payload = self._version.create(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return PlayerStreamerInstance(self._version, payload)

    async def create_async(
        self,
        video=values.unset,
        status_callback=values.unset,
        status_callback_method=values.unset,
        max_duration=values.unset,
    ):
        """
        Asynchronously create the PlayerStreamerInstance

        :param bool video: Specifies whether the PlayerStreamer is configured to stream video. Defaults to `true`.
        :param str status_callback: The URL to which Twilio will send asynchronous webhook requests for every PlayerStreamer event. See [Status Callbacks](/docs/live/status-callbacks) for more details.
        :param str status_callback_method: The HTTP method Twilio should use to call the `status_callback` URL. Can be `POST` or `GET` and the default is `POST`.
        :param int max_duration: The maximum time, in seconds, that the PlayerStreamer is active (`created` or `started`) before automatically ends. The default value is 300 seconds, and the maximum value is 90000 seconds. Once this maximum duration is reached, Twilio will end the PlayerStreamer, regardless of whether media is still streaming.

        :returns: The created PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerInstance
        """
        data = values.of(
            {
                "Video": video,
                "StatusCallback": status_callback,
                "StatusCallbackMethod": status_callback_method,
                "MaxDuration": max_duration,
            }
        )

        payload = await self._version.create_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return PlayerStreamerInstance(self._version, payload)

    def stream(
        self, order=values.unset, status=values.unset, limit=None, page_size=None
    ):
        """
        Streams PlayerStreamerInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param PlayerStreamerInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param PlayerStreamerInstance.Status status: Status to filter by, with possible values `created`, `started`, `ended`, or `failed`.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.player_streamer.PlayerStreamerInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(order=order, status=status, page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(
        self, order=values.unset, status=values.unset, limit=None, page_size=None
    ):
        """
        Asynchronously streams PlayerStreamerInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param PlayerStreamerInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param PlayerStreamerInstance.Status status: Status to filter by, with possible values `created`, `started`, `ended`, or `failed`.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.player_streamer.PlayerStreamerInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(
            order=order, status=status, page_size=limits["page_size"]
        )

        return await self._version.stream_async(page, limits["limit"])

    def list(self, order=values.unset, status=values.unset, limit=None, page_size=None):
        """
        Lists PlayerStreamerInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param PlayerStreamerInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param PlayerStreamerInstance.Status status: Status to filter by, with possible values `created`, `started`, `ended`, or `failed`.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.player_streamer.PlayerStreamerInstance]
        """
        return list(
            self.stream(
                order=order,
                status=status,
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(
        self, order=values.unset, status=values.unset, limit=None, page_size=None
    ):
        """
        Asynchronously lists PlayerStreamerInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param PlayerStreamerInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param PlayerStreamerInstance.Status status: Status to filter by, with possible values `created`, `started`, `ended`, or `failed`.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.player_streamer.PlayerStreamerInstance]
        """
        return list(
            await self.stream_async(
                order=order,
                status=status,
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self,
        order=values.unset,
        status=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Retrieve a single page of PlayerStreamerInstance records from the API.
        Request is executed immediately

        :param PlayerStreamerInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param PlayerStreamerInstance.Status status: Status to filter by, with possible values `created`, `started`, `ended`, or `failed`.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerPage
        """
        data = values.of(
            {
                "Order": order,
                "Status": status,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return PlayerStreamerPage(self._version, response)

    async def page_async(
        self,
        order=values.unset,
        status=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Asynchronously retrieve a single page of PlayerStreamerInstance records from the API.
        Request is executed immediately

        :param PlayerStreamerInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param PlayerStreamerInstance.Status status: Status to filter by, with possible values `created`, `started`, `ended`, or `failed`.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerPage
        """
        data = values.of(
            {
                "Order": order,
                "Status": status,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return PlayerStreamerPage(self._version, response)

    def get_page(self, target_url):
        """
        Retrieve a specific page of PlayerStreamerInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return PlayerStreamerPage(self._version, response)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of PlayerStreamerInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of PlayerStreamerInstance
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return PlayerStreamerPage(self._version, response)

    def get(self, sid):
        """
        Constructs a PlayerStreamerContext

        :param sid: The SID of the PlayerStreamer resource to update.

        :returns: twilio.rest.media.v1.player_streamer.PlayerStreamerContext
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerContext
        """
        return PlayerStreamerContext(self._version, sid=sid)

    def __call__(self, sid):
        """
        Constructs a PlayerStreamerContext

        :param sid: The SID of the PlayerStreamer resource to update.

        :returns: twilio.rest.media.v1.player_streamer.PlayerStreamerContext
        :rtype: twilio.rest.media.v1.player_streamer.PlayerStreamerContext
        """
        return PlayerStreamerContext(self._version, sid=sid)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Media.V1.PlayerStreamerList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Bulkexports
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version


class ExportConfigurationInstance(InstanceResource):
    def __init__(self, version, payload, resource_type: Optional[str] = None):
        """
        Initialize the ExportConfigurationInstance

        :returns: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        """
        super().__init__(version)

        self._properties = {
            "enabled": payload.get("enabled"),
            "webhook_url": payload.get("webhook_url"),
            "webhook_method": payload.get("webhook_method"),
            "resource_type": payload.get("resource_type"),
            "url": payload.get("url"),
        }

        self._solution = {
            "resource_type": resource_type or self._properties["resource_type"],
        }
        self._context: Optional[ExportConfigurationContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: ExportConfigurationContext for this ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationContext
        """
        if self._context is None:
            self._context = ExportConfigurationContext(
                self._version,
                resource_type=self._solution["resource_type"],
            )
        return self._context

    @property
    def enabled(self):
        """
        :returns: If true, Twilio will automatically generate every day's file when the day is over.
        :rtype: bool
        """
        return self._properties["enabled"]

    @property
    def webhook_url(self):
        """
        :returns: Stores the URL destination for the method specified in webhook_method.
        :rtype: str
        """
        return self._properties["webhook_url"]

    @property
    def webhook_method(self):
        """
        :returns: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url
        :rtype: str
        """
        return self._properties["webhook_method"]

    @property
    def resource_type(self):
        """
        :returns: The type of communication – Messages, Calls, Conferences, and Participants
        :rtype: str
        """
        return self._properties["resource_type"]

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: str
        """
        return self._properties["url"]

    def fetch(self):
        """
        Fetch the ExportConfigurationInstance


        :returns: The fetched ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the ExportConfigurationInstance


        :returns: The fetched ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        """
        return await self._proxy.fetch_async()

    def update(
        self,
        enabled=values.unset,
        webhook_url=values.unset,
        webhook_method=values.unset,
    ):
        """
        Update the ExportConfigurationInstance

        :param bool enabled: If true, Twilio will automatically generate every day's file when the day is over.
        :param str webhook_url: Stores the URL destination for the method specified in webhook_method.
        :param str webhook_method: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url

        :returns: The updated ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        """
        return self._proxy.update(
            enabled=enabled,
            webhook_url=webhook_url,
            webhook_method=webhook_method,
        )

    async def update_async(
        self,
        enabled=values.unset,
        webhook_url=values.unset,
        webhook_method=values.unset,
    ):
        """
        Asynchronous coroutine to update the ExportConfigurationInstance

        :param bool enabled: If true, Twilio will automatically generate every day's file when the day is over.
        :param str webhook_url: Stores the URL destination for the method specified in webhook_method.
        :param str webhook_method: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url

        :returns: The updated ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        """
        return await self._proxy.update_async(
            enabled=enabled,
            webhook_url=webhook_url,
            webhook_method=webhook_method,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Bulkexports.V1.ExportConfigurationInstance {}>".format(context)


class ExportConfigurationContext(InstanceContext):
    def __init__(self, version: Version, resource_type: str):
        """
        Initialize the ExportConfigurationContext

        :param Version version: Version that contains the resource
        :param resource_type: The type of communication – Messages, Calls, Conferences, and Participants

        :returns: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationContext
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "resource_type": resource_type,
        }
        self._uri = "/Exports/{resource_type}/Configuration".format(**self._solution)

    def fetch(self):
        """
        Fetch the ExportConfigurationInstance


        :returns: The fetched ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return ExportConfigurationInstance(
            self._version,
            payload,
            resource_type=self._solution["resource_type"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the ExportConfigurationInstance


        :returns: The fetched ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return ExportConfigurationInstance(
            self._version,
            payload,
            resource_type=self._solution["resource_type"],
        )

    def update(
        self,
        enabled=values.unset,
        webhook_url=values.unset,
        webhook_method=values.unset,
    ):
        """
        Update the ExportConfigurationInstance

        :param bool enabled: If true, Twilio will automatically generate every day's file when the day is over.
        :param str webhook_url: Stores the URL destination for the method specified in webhook_method.
        :param str webhook_method: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url

        :returns: The updated ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        """
        data = values.of(
            {
                "Enabled": enabled,
                "WebhookUrl": webhook_url,
                "WebhookMethod": webhook_method,
            }
        )

        payload = self._version.update(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return ExportConfigurationInstance(
            self._version, payload, resource_type=self._solution["resource_type"]
        )

    async def update_async(
        self,
        enabled=values.unset,
        webhook_url=values.unset,
        webhook_method=values.unset,
    ):
        """
        Asynchronous coroutine to update the ExportConfigurationInstance

        :param bool enabled: If true, Twilio will automatically generate every day's file when the day is over.
        :param str webhook_url: Stores the URL destination for the method specified in webhook_method.
        :param str webhook_method: Sets whether Twilio should call a webhook URL when the automatic generation is complete, using GET or POST. The actual destination is set in the webhook_url

        :returns: The updated ExportConfigurationInstance
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationInstance
        """
        data = values.of(
            {
                "Enabled": enabled,
                "WebhookUrl": webhook_url,
                "WebhookMethod": webhook_method,
            }
        )

        payload = await self._version.update_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return ExportConfigurationInstance(
            self._version, payload, resource_type=self._solution["resource_type"]
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Bulkexports.V1.ExportConfigurationContext {}>".format(context)


class ExportConfigurationList(ListResource):
    def __init__(self, version: Version):
        """
        Initialize the ExportConfigurationList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationList
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationList
        """
        super().__init__(version)

    def get(self, resource_type):
        """
        Constructs a ExportConfigurationContext

        :param resource_type: The type of communication – Messages, Calls, Conferences, and Participants

        :returns: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationContext
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationContext
        """
        return ExportConfigurationContext(self._version, resource_type=resource_type)

    def __call__(self, resource_type):
        """
        Constructs a ExportConfigurationContext

        :param resource_type: The type of communication – Messages, Calls, Conferences, and Participants

        :returns: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationContext
        :rtype: twilio.rest.bulkexports.v1.export_configuration.ExportConfigurationContext
        """
        return ExportConfigurationContext(self._version, resource_type=resource_type)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Bulkexports.V1.ExportConfigurationList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Taskrouter
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page
from twilio.rest.taskrouter.v1.workspace.activity import ActivityList
from twilio.rest.taskrouter.v1.workspace.event import EventList
from twilio.rest.taskrouter.v1.workspace.task import TaskList
from twilio.rest.taskrouter.v1.workspace.task_channel import TaskChannelList
from twilio.rest.taskrouter.v1.workspace.task_queue import TaskQueueList
from twilio.rest.taskrouter.v1.workspace.worker import WorkerList
from twilio.rest.taskrouter.v1.workspace.workflow import WorkflowList
from twilio.rest.taskrouter.v1.workspace.workspace_cumulative_statistics import (
    WorkspaceCumulativeStatisticsList,
)
from twilio.rest.taskrouter.v1.workspace.workspace_real_time_statistics import (
    WorkspaceRealTimeStatisticsList,
)
from twilio.rest.taskrouter.v1.workspace.workspace_statistics import (
    WorkspaceStatisticsList,
)


class WorkspaceInstance(InstanceResource):
    class QueueOrder(object):
        FIFO = "FIFO"
        LIFO = "LIFO"

    def __init__(self, version, payload, sid: Optional[str] = None):
        """
        Initialize the WorkspaceInstance

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        super().__init__(version)

        self._properties = {
            "account_sid": payload.get("account_sid"),
            "date_created": deserialize.iso8601_datetime(payload.get("date_created")),
            "date_updated": deserialize.iso8601_datetime(payload.get("date_updated")),
            "default_activity_name": payload.get("default_activity_name"),
            "default_activity_sid": payload.get("default_activity_sid"),
            "event_callback_url": payload.get("event_callback_url"),
            "events_filter": payload.get("events_filter"),
            "friendly_name": payload.get("friendly_name"),
            "multi_task_enabled": payload.get("multi_task_enabled"),
            "sid": payload.get("sid"),
            "timeout_activity_name": payload.get("timeout_activity_name"),
            "timeout_activity_sid": payload.get("timeout_activity_sid"),
            "prioritize_queue_order": payload.get("prioritize_queue_order"),
            "url": payload.get("url"),
            "links": payload.get("links"),
        }

        self._solution = {
            "sid": sid or self._properties["sid"],
        }
        self._context: Optional[WorkspaceContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: WorkspaceContext for this WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceContext
        """
        if self._context is None:
            self._context = WorkspaceContext(
                self._version,
                sid=self._solution["sid"],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the Workspace resource.
        :rtype: str
        """
        return self._properties["account_sid"]

    @property
    def date_created(self):
        """
        :returns: The date and time in GMT when the resource was created specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_created"]

    @property
    def date_updated(self):
        """
        :returns: The date and time in GMT when the resource was last updated specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_updated"]

    @property
    def default_activity_name(self):
        """
        :returns: The name of the default activity.
        :rtype: str
        """
        return self._properties["default_activity_name"]

    @property
    def default_activity_sid(self):
        """
        :returns: The SID of the Activity that will be used when new Workers are created in the Workspace.
        :rtype: str
        """
        return self._properties["default_activity_sid"]

    @property
    def event_callback_url(self):
        """
        :returns: The URL we call when an event occurs. If provided, the Workspace will publish events to this URL, for example, to collect data for reporting. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :rtype: str
        """
        return self._properties["event_callback_url"]

    @property
    def events_filter(self):
        """
        :returns: The list of Workspace events for which to call `event_callback_url`. For example, if `EventsFilter=task.created, task.canceled, worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :rtype: str
        """
        return self._properties["events_filter"]

    @property
    def friendly_name(self):
        """
        :returns: The string that you assigned to describe the Workspace resource. For example `Customer Support` or `2014 Election Campaign`.
        :rtype: str
        """
        return self._properties["friendly_name"]

    @property
    def multi_task_enabled(self):
        """
        :returns: Whether multi-tasking is enabled. The default is `true`, which enables multi-tasking. Multi-tasking allows Workers to handle multiple Tasks simultaneously. When enabled (`true`), each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking each Worker would only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :rtype: bool
        """
        return self._properties["multi_task_enabled"]

    @property
    def sid(self):
        """
        :returns: The unique string that we created to identify the Workspace resource.
        :rtype: str
        """
        return self._properties["sid"]

    @property
    def timeout_activity_name(self):
        """
        :returns: The name of the timeout activity.
        :rtype: str
        """
        return self._properties["timeout_activity_name"]

    @property
    def timeout_activity_sid(self):
        """
        :returns: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
        :rtype: str
        """
        return self._properties["timeout_activity_sid"]

    @property
    def prioritize_queue_order(self):
        """
        :returns:
        :rtype: WorkspaceInstance.QueueOrder
        """
        return self._properties["prioritize_queue_order"]

    @property
    def url(self):
        """
        :returns: The absolute URL of the Workspace resource.
        :rtype: str
        """
        return self._properties["url"]

    @property
    def links(self):
        """
        :returns: The URLs of related resources.
        :rtype: dict
        """
        return self._properties["links"]

    def delete(self):
        """
        Deletes the WorkspaceInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    async def delete_async(self):
        """
        Asynchronous coroutine that deletes the WorkspaceInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return await self._proxy.delete_async()

    def fetch(self):
        """
        Fetch the WorkspaceInstance


        :returns: The fetched WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the WorkspaceInstance


        :returns: The fetched WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        return await self._proxy.fetch_async()

    def update(
        self,
        default_activity_sid=values.unset,
        event_callback_url=values.unset,
        events_filter=values.unset,
        friendly_name=values.unset,
        multi_task_enabled=values.unset,
        timeout_activity_sid=values.unset,
        prioritize_queue_order=values.unset,
    ):
        """
        Update the WorkspaceInstance

        :param str default_activity_sid: The SID of the Activity that will be used when new Workers are created in the Workspace.
        :param str event_callback_url: The URL we should call when an event occurs. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param str events_filter: The list of Workspace events for which to call event_callback_url. For example if `EventsFilter=task.created,task.canceled,worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param str friendly_name: A descriptive string that you create to describe the Workspace resource. For example: `Sales Call Center` or `Customer Support Team`.
        :param bool multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be maintained as multi-tasking. There is no default when omitting this parameter. A multi-tasking Workspace can't be updated to single-tasking unless it is not a Flex Project and another (legacy) single-tasking Workspace exists. Multi-tasking allows Workers to handle multiple Tasks simultaneously. In multi-tasking mode, each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param str timeout_activity_sid: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
        :param WorkspaceInstance.QueueOrder prioritize_queue_order:

        :returns: The updated WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        return self._proxy.update(
            default_activity_sid=default_activity_sid,
            event_callback_url=event_callback_url,
            events_filter=events_filter,
            friendly_name=friendly_name,
            multi_task_enabled=multi_task_enabled,
            timeout_activity_sid=timeout_activity_sid,
            prioritize_queue_order=prioritize_queue_order,
        )

    async def update_async(
        self,
        default_activity_sid=values.unset,
        event_callback_url=values.unset,
        events_filter=values.unset,
        friendly_name=values.unset,
        multi_task_enabled=values.unset,
        timeout_activity_sid=values.unset,
        prioritize_queue_order=values.unset,
    ):
        """
        Asynchronous coroutine to update the WorkspaceInstance

        :param str default_activity_sid: The SID of the Activity that will be used when new Workers are created in the Workspace.
        :param str event_callback_url: The URL we should call when an event occurs. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param str events_filter: The list of Workspace events for which to call event_callback_url. For example if `EventsFilter=task.created,task.canceled,worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param str friendly_name: A descriptive string that you create to describe the Workspace resource. For example: `Sales Call Center` or `Customer Support Team`.
        :param bool multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be maintained as multi-tasking. There is no default when omitting this parameter. A multi-tasking Workspace can't be updated to single-tasking unless it is not a Flex Project and another (legacy) single-tasking Workspace exists. Multi-tasking allows Workers to handle multiple Tasks simultaneously. In multi-tasking mode, each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param str timeout_activity_sid: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
        :param WorkspaceInstance.QueueOrder prioritize_queue_order:

        :returns: The updated WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        return await self._proxy.update_async(
            default_activity_sid=default_activity_sid,
            event_callback_url=event_callback_url,
            events_filter=events_filter,
            friendly_name=friendly_name,
            multi_task_enabled=multi_task_enabled,
            timeout_activity_sid=timeout_activity_sid,
            prioritize_queue_order=prioritize_queue_order,
        )

    @property
    def activities(self):
        """
        Access the activities

        :returns: twilio.rest.taskrouter.v1.workspace.ActivityList
        :rtype: twilio.rest.taskrouter.v1.workspace.ActivityList
        """
        return self._proxy.activities

    @property
    def events(self):
        """
        Access the events

        :returns: twilio.rest.taskrouter.v1.workspace.EventList
        :rtype: twilio.rest.taskrouter.v1.workspace.EventList
        """
        return self._proxy.events

    @property
    def tasks(self):
        """
        Access the tasks

        :returns: twilio.rest.taskrouter.v1.workspace.TaskList
        :rtype: twilio.rest.taskrouter.v1.workspace.TaskList
        """
        return self._proxy.tasks

    @property
    def task_channels(self):
        """
        Access the task_channels

        :returns: twilio.rest.taskrouter.v1.workspace.TaskChannelList
        :rtype: twilio.rest.taskrouter.v1.workspace.TaskChannelList
        """
        return self._proxy.task_channels

    @property
    def task_queues(self):
        """
        Access the task_queues

        :returns: twilio.rest.taskrouter.v1.workspace.TaskQueueList
        :rtype: twilio.rest.taskrouter.v1.workspace.TaskQueueList
        """
        return self._proxy.task_queues

    @property
    def workers(self):
        """
        Access the workers

        :returns: twilio.rest.taskrouter.v1.workspace.WorkerList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkerList
        """
        return self._proxy.workers

    @property
    def workflows(self):
        """
        Access the workflows

        :returns: twilio.rest.taskrouter.v1.workspace.WorkflowList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkflowList
        """
        return self._proxy.workflows

    @property
    def cumulative_statistics(self):
        """
        Access the cumulative_statistics

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceCumulativeStatisticsList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceCumulativeStatisticsList
        """
        return self._proxy.cumulative_statistics

    @property
    def real_time_statistics(self):
        """
        Access the real_time_statistics

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceRealTimeStatisticsList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceRealTimeStatisticsList
        """
        return self._proxy.real_time_statistics

    @property
    def statistics(self):
        """
        Access the statistics

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceStatisticsList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceStatisticsList
        """
        return self._proxy.statistics

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Taskrouter.V1.WorkspaceInstance {}>".format(context)


class WorkspaceContext(InstanceContext):
    def __init__(self, version: Version, sid: str):
        """
        Initialize the WorkspaceContext

        :param Version version: Version that contains the resource
        :param sid: The SID of the Workspace resource to update.

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceContext
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "sid": sid,
        }
        self._uri = "/Workspaces/{sid}".format(**self._solution)

        self._activities: Optional[ActivityList] = None
        self._events: Optional[EventList] = None
        self._tasks: Optional[TaskList] = None
        self._task_channels: Optional[TaskChannelList] = None
        self._task_queues: Optional[TaskQueueList] = None
        self._workers: Optional[WorkerList] = None
        self._workflows: Optional[WorkflowList] = None
        self._cumulative_statistics: Optional[WorkspaceCumulativeStatisticsList] = None
        self._real_time_statistics: Optional[WorkspaceRealTimeStatisticsList] = None
        self._statistics: Optional[WorkspaceStatisticsList] = None

    def delete(self):
        """
        Deletes the WorkspaceInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(
            method="DELETE",
            uri=self._uri,
        )

    async def delete_async(self):
        """
        Asynchronous coroutine that deletes the WorkspaceInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return await self._version.delete_async(
            method="DELETE",
            uri=self._uri,
        )

    def fetch(self):
        """
        Fetch the WorkspaceInstance


        :returns: The fetched WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return WorkspaceInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the WorkspaceInstance


        :returns: The fetched WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return WorkspaceInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    def update(
        self,
        default_activity_sid=values.unset,
        event_callback_url=values.unset,
        events_filter=values.unset,
        friendly_name=values.unset,
        multi_task_enabled=values.unset,
        timeout_activity_sid=values.unset,
        prioritize_queue_order=values.unset,
    ):
        """
        Update the WorkspaceInstance

        :param str default_activity_sid: The SID of the Activity that will be used when new Workers are created in the Workspace.
        :param str event_callback_url: The URL we should call when an event occurs. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param str events_filter: The list of Workspace events for which to call event_callback_url. For example if `EventsFilter=task.created,task.canceled,worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param str friendly_name: A descriptive string that you create to describe the Workspace resource. For example: `Sales Call Center` or `Customer Support Team`.
        :param bool multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be maintained as multi-tasking. There is no default when omitting this parameter. A multi-tasking Workspace can't be updated to single-tasking unless it is not a Flex Project and another (legacy) single-tasking Workspace exists. Multi-tasking allows Workers to handle multiple Tasks simultaneously. In multi-tasking mode, each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param str timeout_activity_sid: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
        :param WorkspaceInstance.QueueOrder prioritize_queue_order:

        :returns: The updated WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        data = values.of(
            {
                "DefaultActivitySid": default_activity_sid,
                "EventCallbackUrl": event_callback_url,
                "EventsFilter": events_filter,
                "FriendlyName": friendly_name,
                "MultiTaskEnabled": multi_task_enabled,
                "TimeoutActivitySid": timeout_activity_sid,
                "PrioritizeQueueOrder": prioritize_queue_order,
            }
        )

        payload = self._version.update(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return WorkspaceInstance(self._version, payload, sid=self._solution["sid"])

    async def update_async(
        self,
        default_activity_sid=values.unset,
        event_callback_url=values.unset,
        events_filter=values.unset,
        friendly_name=values.unset,
        multi_task_enabled=values.unset,
        timeout_activity_sid=values.unset,
        prioritize_queue_order=values.unset,
    ):
        """
        Asynchronous coroutine to update the WorkspaceInstance

        :param str default_activity_sid: The SID of the Activity that will be used when new Workers are created in the Workspace.
        :param str event_callback_url: The URL we should call when an event occurs. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param str events_filter: The list of Workspace events for which to call event_callback_url. For example if `EventsFilter=task.created,task.canceled,worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param str friendly_name: A descriptive string that you create to describe the Workspace resource. For example: `Sales Call Center` or `Customer Support Team`.
        :param bool multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be maintained as multi-tasking. There is no default when omitting this parameter. A multi-tasking Workspace can't be updated to single-tasking unless it is not a Flex Project and another (legacy) single-tasking Workspace exists. Multi-tasking allows Workers to handle multiple Tasks simultaneously. In multi-tasking mode, each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param str timeout_activity_sid: The SID of the Activity that will be assigned to a Worker when a Task reservation times out without a response.
        :param WorkspaceInstance.QueueOrder prioritize_queue_order:

        :returns: The updated WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        data = values.of(
            {
                "DefaultActivitySid": default_activity_sid,
                "EventCallbackUrl": event_callback_url,
                "EventsFilter": events_filter,
                "FriendlyName": friendly_name,
                "MultiTaskEnabled": multi_task_enabled,
                "TimeoutActivitySid": timeout_activity_sid,
                "PrioritizeQueueOrder": prioritize_queue_order,
            }
        )

        payload = await self._version.update_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return WorkspaceInstance(self._version, payload, sid=self._solution["sid"])

    @property
    def activities(self):
        """
        Access the activities

        :returns: twilio.rest.taskrouter.v1.workspace.ActivityList
        :rtype: twilio.rest.taskrouter.v1.workspace.ActivityList
        """
        if self._activities is None:
            self._activities = ActivityList(
                self._version,
                self._solution["sid"],
            )
        return self._activities

    @property
    def events(self):
        """
        Access the events

        :returns: twilio.rest.taskrouter.v1.workspace.EventList
        :rtype: twilio.rest.taskrouter.v1.workspace.EventList
        """
        if self._events is None:
            self._events = EventList(
                self._version,
                self._solution["sid"],
            )
        return self._events

    @property
    def tasks(self):
        """
        Access the tasks

        :returns: twilio.rest.taskrouter.v1.workspace.TaskList
        :rtype: twilio.rest.taskrouter.v1.workspace.TaskList
        """
        if self._tasks is None:
            self._tasks = TaskList(
                self._version,
                self._solution["sid"],
            )
        return self._tasks

    @property
    def task_channels(self):
        """
        Access the task_channels

        :returns: twilio.rest.taskrouter.v1.workspace.TaskChannelList
        :rtype: twilio.rest.taskrouter.v1.workspace.TaskChannelList
        """
        if self._task_channels is None:
            self._task_channels = TaskChannelList(
                self._version,
                self._solution["sid"],
            )
        return self._task_channels

    @property
    def task_queues(self):
        """
        Access the task_queues

        :returns: twilio.rest.taskrouter.v1.workspace.TaskQueueList
        :rtype: twilio.rest.taskrouter.v1.workspace.TaskQueueList
        """
        if self._task_queues is None:
            self._task_queues = TaskQueueList(
                self._version,
                self._solution["sid"],
            )
        return self._task_queues

    @property
    def workers(self):
        """
        Access the workers

        :returns: twilio.rest.taskrouter.v1.workspace.WorkerList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkerList
        """
        if self._workers is None:
            self._workers = WorkerList(
                self._version,
                self._solution["sid"],
            )
        return self._workers

    @property
    def workflows(self):
        """
        Access the workflows

        :returns: twilio.rest.taskrouter.v1.workspace.WorkflowList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkflowList
        """
        if self._workflows is None:
            self._workflows = WorkflowList(
                self._version,
                self._solution["sid"],
            )
        return self._workflows

    @property
    def cumulative_statistics(self):
        """
        Access the cumulative_statistics

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceCumulativeStatisticsList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceCumulativeStatisticsList
        """
        if self._cumulative_statistics is None:
            self._cumulative_statistics = WorkspaceCumulativeStatisticsList(
                self._version,
                self._solution["sid"],
            )
        return self._cumulative_statistics

    @property
    def real_time_statistics(self):
        """
        Access the real_time_statistics

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceRealTimeStatisticsList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceRealTimeStatisticsList
        """
        if self._real_time_statistics is None:
            self._real_time_statistics = WorkspaceRealTimeStatisticsList(
                self._version,
                self._solution["sid"],
            )
        return self._real_time_statistics

    @property
    def statistics(self):
        """
        Access the statistics

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceStatisticsList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceStatisticsList
        """
        if self._statistics is None:
            self._statistics = WorkspaceStatisticsList(
                self._version,
                self._solution["sid"],
            )
        return self._statistics

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Taskrouter.V1.WorkspaceContext {}>".format(context)


class WorkspacePage(Page):
    def get_instance(self, payload):
        """
        Build an instance of WorkspaceInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        return WorkspaceInstance(self._version, payload)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Taskrouter.V1.WorkspacePage>"


class WorkspaceList(ListResource):
    def __init__(self, version: Version):
        """
        Initialize the WorkspaceList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceList
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceList
        """
        super().__init__(version)

        self._uri = "/Workspaces"

    def create(
        self,
        friendly_name,
        event_callback_url=values.unset,
        events_filter=values.unset,
        multi_task_enabled=values.unset,
        template=values.unset,
        prioritize_queue_order=values.unset,
    ):
        """
        Create the WorkspaceInstance

        :param str friendly_name: A descriptive string that you create to describe the Workspace resource. It can be up to 64 characters long. For example: `Customer Support` or `2014 Election Campaign`.
        :param str event_callback_url: The URL we should call when an event occurs. If provided, the Workspace will publish events to this URL, for example, to collect data for reporting. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param str events_filter: The list of Workspace events for which to call event_callback_url. For example, if `EventsFilter=task.created, task.canceled, worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param bool multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be created as multi-tasking. The default is `true`. Multi-tasking allows Workers to handle multiple Tasks simultaneously. When enabled (`true`), each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param str template: An available template name. Can be: `NONE` or `FIFO` and the default is `NONE`. Pre-configures the Workspace with the Workflow and Activities specified in the template. `NONE` will create a Workspace with only a set of default activities. `FIFO` will configure TaskRouter with a set of default activities and a single TaskQueue for first-in, first-out distribution, which can be useful when you are getting started with TaskRouter.
        :param WorkspaceInstance.QueueOrder prioritize_queue_order:

        :returns: The created WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        data = values.of(
            {
                "FriendlyName": friendly_name,
                "EventCallbackUrl": event_callback_url,
                "EventsFilter": events_filter,
                "MultiTaskEnabled": multi_task_enabled,
                "Template": template,
                "PrioritizeQueueOrder": prioritize_queue_order,
            }
        )

        payload = self._version.create(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return WorkspaceInstance(self._version, payload)

    async def create_async(
        self,
        friendly_name,
        event_callback_url=values.unset,
        events_filter=values.unset,
        multi_task_enabled=values.unset,
        template=values.unset,
        prioritize_queue_order=values.unset,
    ):
        """
        Asynchronously create the WorkspaceInstance

        :param str friendly_name: A descriptive string that you create to describe the Workspace resource. It can be up to 64 characters long. For example: `Customer Support` or `2014 Election Campaign`.
        :param str event_callback_url: The URL we should call when an event occurs. If provided, the Workspace will publish events to this URL, for example, to collect data for reporting. See [Workspace Events](https://www.twilio.com/docs/taskrouter/api/event) for more information. This parameter supports Twilio's [Webhooks (HTTP callbacks) Connection Overrides](https://www.twilio.com/docs/usage/webhooks/webhooks-connection-overrides).
        :param str events_filter: The list of Workspace events for which to call event_callback_url. For example, if `EventsFilter=task.created, task.canceled, worker.activity.update`, then TaskRouter will call event_callback_url only when a task is created, canceled, or a Worker activity is updated.
        :param bool multi_task_enabled: Whether to enable multi-tasking. Can be: `true` to enable multi-tasking, or `false` to disable it. However, all workspaces should be created as multi-tasking. The default is `true`. Multi-tasking allows Workers to handle multiple Tasks simultaneously. When enabled (`true`), each Worker can receive parallel reservations up to the per-channel maximums defined in the Workers section. In single-tasking mode (legacy mode), each Worker will only receive a new reservation when the previous task is completed. Learn more at [Multitasking](https://www.twilio.com/docs/taskrouter/multitasking).
        :param str template: An available template name. Can be: `NONE` or `FIFO` and the default is `NONE`. Pre-configures the Workspace with the Workflow and Activities specified in the template. `NONE` will create a Workspace with only a set of default activities. `FIFO` will configure TaskRouter with a set of default activities and a single TaskQueue for first-in, first-out distribution, which can be useful when you are getting started with TaskRouter.
        :param WorkspaceInstance.QueueOrder prioritize_queue_order:

        :returns: The created WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceInstance
        """
        data = values.of(
            {
                "FriendlyName": friendly_name,
                "EventCallbackUrl": event_callback_url,
                "EventsFilter": events_filter,
                "MultiTaskEnabled": multi_task_enabled,
                "Template": template,
                "PrioritizeQueueOrder": prioritize_queue_order,
            }
        )

        payload = await self._version.create_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return WorkspaceInstance(self._version, payload)

    def stream(self, friendly_name=values.unset, limit=None, page_size=None):
        """
        Streams WorkspaceInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.taskrouter.v1.workspace.WorkspaceInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(friendly_name=friendly_name, page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(
        self, friendly_name=values.unset, limit=None, page_size=None
    ):
        """
        Asynchronously streams WorkspaceInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.taskrouter.v1.workspace.WorkspaceInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(
            friendly_name=friendly_name, page_size=limits["page_size"]
        )

        return await self._version.stream_async(page, limits["limit"])

    def list(self, friendly_name=values.unset, limit=None, page_size=None):
        """
        Lists WorkspaceInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.taskrouter.v1.workspace.WorkspaceInstance]
        """
        return list(
            self.stream(
                friendly_name=friendly_name,
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(self, friendly_name=values.unset, limit=None, page_size=None):
        """
        Asynchronously lists WorkspaceInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.taskrouter.v1.workspace.WorkspaceInstance]
        """
        return list(
            await self.stream_async(
                friendly_name=friendly_name,
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self,
        friendly_name=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Retrieve a single page of WorkspaceInstance records from the API.
        Request is executed immediately

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspacePage
        """
        data = values.of(
            {
                "FriendlyName": friendly_name,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return WorkspacePage(self._version, response)

    async def page_async(
        self,
        friendly_name=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Asynchronously retrieve a single page of WorkspaceInstance records from the API.
        Request is executed immediately

        :param str friendly_name: The `friendly_name` of the Workspace resources to read. For example `Customer Support` or `2014 Election Campaign`.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspacePage
        """
        data = values.of(
            {
                "FriendlyName": friendly_name,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return WorkspacePage(self._version, response)

    def get_page(self, target_url):
        """
        Retrieve a specific page of WorkspaceInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspacePage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return WorkspacePage(self._version, response)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of WorkspaceInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of WorkspaceInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspacePage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return WorkspacePage(self._version, response)

    def get(self, sid):
        """
        Constructs a WorkspaceContext

        :param sid: The SID of the Workspace resource to update.

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceContext
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceContext
        """
        return WorkspaceContext(self._version, sid=sid)

    def __call__(self, sid):
        """
        Constructs a WorkspaceContext

        :param sid: The SID of the Workspace resource to update.

        :returns: twilio.rest.taskrouter.v1.workspace.WorkspaceContext
        :rtype: twilio.rest.taskrouter.v1.workspace.WorkspaceContext
        """
        return WorkspaceContext(self._version, sid=sid)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Taskrouter.V1.WorkspaceList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Wireless
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from twilio.base import deserialize, values

from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page


class DataSessionInstance(InstanceResource):
    def __init__(self, version, payload, sim_sid: str):
        """
        Initialize the DataSessionInstance

        :returns: twilio.rest.wireless.v1.sim.data_session.DataSessionInstance
        :rtype: twilio.rest.wireless.v1.sim.data_session.DataSessionInstance
        """
        super().__init__(version)

        self._properties = {
            "sid": payload.get("sid"),
            "sim_sid": payload.get("sim_sid"),
            "account_sid": payload.get("account_sid"),
            "radio_link": payload.get("radio_link"),
            "operator_mcc": payload.get("operator_mcc"),
            "operator_mnc": payload.get("operator_mnc"),
            "operator_country": payload.get("operator_country"),
            "operator_name": payload.get("operator_name"),
            "cell_id": payload.get("cell_id"),
            "cell_location_estimate": payload.get("cell_location_estimate"),
            "packets_uploaded": deserialize.integer(payload.get("packets_uploaded")),
            "packets_downloaded": deserialize.integer(
                payload.get("packets_downloaded")
            ),
            "last_updated": deserialize.iso8601_datetime(payload.get("last_updated")),
            "start": deserialize.iso8601_datetime(payload.get("start")),
            "end": deserialize.iso8601_datetime(payload.get("end")),
            "imei": payload.get("imei"),
        }

        self._solution = {
            "sim_sid": sim_sid,
        }

    @property
    def sid(self):
        """
        :returns: The unique string that we created to identify the DataSession resource.
        :rtype: str
        """
        return self._properties["sid"]

    @property
    def sim_sid(self):
        """
        :returns: The SID of the [Sim resource](https://www.twilio.com/docs/wireless/api/sim-resource) that the Data Session is for.
        :rtype: str
        """
        return self._properties["sim_sid"]

    @property
    def account_sid(self):
        """
        :returns: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the DataSession resource.
        :rtype: str
        """
        return self._properties["account_sid"]

    @property
    def radio_link(self):
        """
        :returns: The generation of wireless technology that the device was using.
        :rtype: str
        """
        return self._properties["radio_link"]

    @property
    def operator_mcc(self):
        """
        :returns: The 'mobile country code' is the unique ID of the home country where the Data Session took place. See: [MCC/MNC lookup](http://mcc-mnc.com/).
        :rtype: str
        """
        return self._properties["operator_mcc"]

    @property
    def operator_mnc(self):
        """
        :returns: The 'mobile network code' is the unique ID specific to the mobile operator network where the Data Session took place.
        :rtype: str
        """
        return self._properties["operator_mnc"]

    @property
    def operator_country(self):
        """
        :returns: The three letter country code representing where the device's Data Session took place. This is determined by looking up the `operator_mcc`.
        :rtype: str
        """
        return self._properties["operator_country"]

    @property
    def operator_name(self):
        """
        :returns: The friendly name of the mobile operator network that the [SIM](https://www.twilio.com/docs/wireless/api/sim-resource)-connected device is attached to. This is determined by looking up the `operator_mnc`.
        :rtype: str
        """
        return self._properties["operator_name"]

    @property
    def cell_id(self):
        """
        :returns: The unique ID of the cellular tower that the device was attached to at the moment when the Data Session was last updated.
        :rtype: str
        """
        return self._properties["cell_id"]

    @property
    def cell_location_estimate(self):
        """
        :returns: An object that describes the estimated location in latitude and longitude where the device's Data Session took place. The location is derived from the `cell_id` when the Data Session was last updated. See [Cell Location Estimate Object](https://www.twilio.com/docs/wireless/api/datasession-resource#cell-location-estimate-object).
        :rtype: dict
        """
        return self._properties["cell_location_estimate"]

    @property
    def packets_uploaded(self):
        """
        :returns: The number of packets uploaded by the device between the `start` time and when the Data Session was last updated.
        :rtype: int
        """
        return self._properties["packets_uploaded"]

    @property
    def packets_downloaded(self):
        """
        :returns: The number of packets downloaded by the device between the `start` time and when the Data Session was last updated.
        :rtype: int
        """
        return self._properties["packets_downloaded"]

    @property
    def last_updated(self):
        """
        :returns: The date that the resource was last updated, given as GMT in [ISO 8601](https://www.iso.org/iso-8601-date-and-time-format.html) format.
        :rtype: datetime
        """
        return self._properties["last_updated"]

    @property
    def start(self):
        """
        :returns: The date that the Data Session started, given as GMT in [ISO 8601](https://www.iso.org/iso-8601-date-and-time-format.html) format.
        :rtype: datetime
        """
        return self._properties["start"]

    @property
    def end(self):
        """
        :returns: The date that the record ended, given as GMT in [ISO 8601](https://www.iso.org/iso-8601-date-and-time-format.html) format.
        :rtype: datetime
        """
        return self._properties["end"]

    @property
    def imei(self):
        """
        :returns: The 'international mobile equipment identity' is the unique ID of the device using the SIM to connect. An IMEI is a 15-digit string: 14 digits for the device identifier plus a check digit calculated using the Luhn formula.
        :rtype: str
        """
        return self._properties["imei"]

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Wireless.V1.DataSessionInstance {}>".format(context)


class DataSessionPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of DataSessionInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.wireless.v1.sim.data_session.DataSessionInstance
        :rtype: twilio.rest.wireless.v1.sim.data_session.DataSessionInstance
        """
        return DataSessionInstance(
            self._version, payload, sim_sid=self._solution["sim_sid"]
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Wireless.V1.DataSessionPage>"


class DataSessionList(ListResource):
    def __init__(self, version: Version, sim_sid: str):
        """
        Initialize the DataSessionList

        :param Version version: Version that contains the resource
        :param sim_sid: The SID of the [Sim resource](https://www.twilio.com/docs/wireless/api/sim-resource) with the Data Sessions to read.

        :returns: twilio.rest.wireless.v1.sim.data_session.DataSessionList
        :rtype: twilio.rest.wireless.v1.sim.data_session.DataSessionList
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "sim_sid": sim_sid,
        }
        self._uri = "/Sims/{sim_sid}/DataSessions".format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams DataSessionInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.wireless.v1.sim.data_session.DataSessionInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(self, limit=None, page_size=None):
        """
        Asynchronously streams DataSessionInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.wireless.v1.sim.data_session.DataSessionInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(page_size=limits["page_size"])

        return await self._version.stream_async(page, limits["limit"])

    def list(self, limit=None, page_size=None):
        """
        Lists DataSessionInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.wireless.v1.sim.data_session.DataSessionInstance]
        """
        return list(
            self.stream(
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(self, limit=None, page_size=None):
        """
        Asynchronously lists DataSessionInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.wireless.v1.sim.data_session.DataSessionInstance]
        """
        return list(
            await self.stream_async(
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Retrieve a single page of DataSessionInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of DataSessionInstance
        :rtype: twilio.rest.wireless.v1.sim.data_session.DataSessionPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return DataSessionPage(self._version, response, self._solution)

    async def page_async(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Asynchronously retrieve a single page of DataSessionInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of DataSessionInstance
        :rtype: twilio.rest.wireless.v1.sim.data_session.DataSessionPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return DataSessionPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of DataSessionInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of DataSessionInstance
        :rtype: twilio.rest.wireless.v1.sim.data_session.DataSessionPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return DataSessionPage(self._version, response, self._solution)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of DataSessionInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of DataSessionInstance
        :rtype: twilio.rest.wireless.v1.sim.data_session.DataSessionPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return DataSessionPage(self._version, response, self._solution)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Wireless.V1.DataSessionList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Messaging
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page
from twilio.rest.messaging.v1.brand_registration.brand_vetting import BrandVettingList


class BrandRegistrationInstance(InstanceResource):
    class BrandFeedback(object):
        TAX_ID = "TAX_ID"
        STOCK_SYMBOL = "STOCK_SYMBOL"
        NONPROFIT = "NONPROFIT"
        GOVERNMENT_ENTITY = "GOVERNMENT_ENTITY"
        OTHERS = "OTHERS"

    class IdentityStatus(object):
        SELF_DECLARED = "SELF_DECLARED"
        UNVERIFIED = "UNVERIFIED"
        VERIFIED = "VERIFIED"
        VETTED_VERIFIED = "VETTED_VERIFIED"

    class Status(object):
        PENDING = "PENDING"
        APPROVED = "APPROVED"
        FAILED = "FAILED"
        IN_REVIEW = "IN_REVIEW"
        DELETED = "DELETED"

    def __init__(self, version, payload, sid: Optional[str] = None):
        """
        Initialize the BrandRegistrationInstance

        :returns: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        super().__init__(version)

        self._properties = {
            "sid": payload.get("sid"),
            "account_sid": payload.get("account_sid"),
            "customer_profile_bundle_sid": payload.get("customer_profile_bundle_sid"),
            "a2p_profile_bundle_sid": payload.get("a2p_profile_bundle_sid"),
            "date_created": deserialize.iso8601_datetime(payload.get("date_created")),
            "date_updated": deserialize.iso8601_datetime(payload.get("date_updated")),
            "brand_type": payload.get("brand_type"),
            "status": payload.get("status"),
            "tcr_id": payload.get("tcr_id"),
            "failure_reason": payload.get("failure_reason"),
            "url": payload.get("url"),
            "brand_score": deserialize.integer(payload.get("brand_score")),
            "brand_feedback": payload.get("brand_feedback"),
            "identity_status": payload.get("identity_status"),
            "russell_3000": payload.get("russell_3000"),
            "government_entity": payload.get("government_entity"),
            "tax_exempt_status": payload.get("tax_exempt_status"),
            "skip_automatic_sec_vet": payload.get("skip_automatic_sec_vet"),
            "mock": payload.get("mock"),
            "links": payload.get("links"),
        }

        self._solution = {
            "sid": sid or self._properties["sid"],
        }
        self._context: Optional[BrandRegistrationContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: BrandRegistrationContext for this BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationContext
        """
        if self._context is None:
            self._context = BrandRegistrationContext(
                self._version,
                sid=self._solution["sid"],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string to identify Brand Registration.
        :rtype: str
        """
        return self._properties["sid"]

    @property
    def account_sid(self):
        """
        :returns: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the Brand Registration resource.
        :rtype: str
        """
        return self._properties["account_sid"]

    @property
    def customer_profile_bundle_sid(self):
        """
        :returns: A2P Messaging Profile Bundle BundleSid.
        :rtype: str
        """
        return self._properties["customer_profile_bundle_sid"]

    @property
    def a2p_profile_bundle_sid(self):
        """
        :returns: A2P Messaging Profile Bundle BundleSid.
        :rtype: str
        """
        return self._properties["a2p_profile_bundle_sid"]

    @property
    def date_created(self):
        """
        :returns: The date and time in GMT when the resource was created specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_created"]

    @property
    def date_updated(self):
        """
        :returns: The date and time in GMT when the resource was last updated specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_updated"]

    @property
    def brand_type(self):
        """
        :returns: Type of brand. One of: \"STANDARD\", \"STARTER\". STARTER is for the low volume, STARTER campaign use case. There can only be one STARTER campaign created per STARTER brand. STANDARD is for all other campaign use cases. Multiple campaign use cases can be created per STANDARD brand.
        :rtype: str
        """
        return self._properties["brand_type"]

    @property
    def status(self):
        """
        :returns:
        :rtype: BrandRegistrationInstance.Status
        """
        return self._properties["status"]

    @property
    def tcr_id(self):
        """
        :returns: Campaign Registry (TCR) Brand ID. Assigned only after successful brand registration.
        :rtype: str
        """
        return self._properties["tcr_id"]

    @property
    def failure_reason(self):
        """
        :returns: A reason why brand registration has failed. Only applicable when status is FAILED.
        :rtype: str
        """
        return self._properties["failure_reason"]

    @property
    def url(self):
        """
        :returns: The absolute URL of the Brand Registration resource.
        :rtype: str
        """
        return self._properties["url"]

    @property
    def brand_score(self):
        """
        :returns: The secondary vetting score if it was done. Otherwise, it will be the brand score if it's returned from TCR. It may be null if no score is available.
        :rtype: int
        """
        return self._properties["brand_score"]

    @property
    def brand_feedback(self):
        """
        :returns: Feedback on how to improve brand score
        :rtype: list[BrandRegistrationInstance.BrandFeedback]
        """
        return self._properties["brand_feedback"]

    @property
    def identity_status(self):
        """
        :returns:
        :rtype: BrandRegistrationInstance.IdentityStatus
        """
        return self._properties["identity_status"]

    @property
    def russell_3000(self):
        """
        :returns: Publicly traded company identified in the Russell 3000 Index
        :rtype: bool
        """
        return self._properties["russell_3000"]

    @property
    def government_entity(self):
        """
        :returns: Identified as a government entity
        :rtype: bool
        """
        return self._properties["government_entity"]

    @property
    def tax_exempt_status(self):
        """
        :returns: Nonprofit organization tax-exempt status per section 501 of the U.S. tax code.
        :rtype: str
        """
        return self._properties["tax_exempt_status"]

    @property
    def skip_automatic_sec_vet(self):
        """
        :returns: A flag to disable automatic secondary vetting for brands which it would otherwise be done.
        :rtype: bool
        """
        return self._properties["skip_automatic_sec_vet"]

    @property
    def mock(self):
        """
        :returns: A boolean that specifies whether brand should be a mock or not. If true, brand will be registered as a mock brand. Defaults to false if no value is provided.
        :rtype: bool
        """
        return self._properties["mock"]

    @property
    def links(self):
        """
        :returns:
        :rtype: dict
        """
        return self._properties["links"]

    def fetch(self):
        """
        Fetch the BrandRegistrationInstance


        :returns: The fetched BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the BrandRegistrationInstance


        :returns: The fetched BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        return await self._proxy.fetch_async()

    def update(self):
        """
        Update the BrandRegistrationInstance


        :returns: The updated BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        return self._proxy.update()

    async def update_async(self):
        """
        Asynchronous coroutine to update the BrandRegistrationInstance


        :returns: The updated BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        return await self._proxy.update_async()

    @property
    def brand_vettings(self):
        """
        Access the brand_vettings

        :returns: twilio.rest.messaging.v1.brand_registration.BrandVettingList
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandVettingList
        """
        return self._proxy.brand_vettings

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Messaging.V1.BrandRegistrationInstance {}>".format(context)


class BrandRegistrationContext(InstanceContext):
    def __init__(self, version: Version, sid: str):
        """
        Initialize the BrandRegistrationContext

        :param Version version: Version that contains the resource
        :param sid: The SID of the Brand Registration resource to update.

        :returns: twilio.rest.messaging.v1.brand_registration.BrandRegistrationContext
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "sid": sid,
        }
        self._uri = "/a2p/BrandRegistrations/{sid}".format(**self._solution)

        self._brand_vettings: Optional[BrandVettingList] = None

    def fetch(self):
        """
        Fetch the BrandRegistrationInstance


        :returns: The fetched BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return BrandRegistrationInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the BrandRegistrationInstance


        :returns: The fetched BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return BrandRegistrationInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    def update(self):
        """
        Update the BrandRegistrationInstance


        :returns: The updated BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        data = values.of({})

        payload = self._version.update(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return BrandRegistrationInstance(
            self._version, payload, sid=self._solution["sid"]
        )

    async def update_async(self):
        """
        Asynchronous coroutine to update the BrandRegistrationInstance


        :returns: The updated BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        data = values.of({})

        payload = await self._version.update_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return BrandRegistrationInstance(
            self._version, payload, sid=self._solution["sid"]
        )

    @property
    def brand_vettings(self):
        """
        Access the brand_vettings

        :returns: twilio.rest.messaging.v1.brand_registration.BrandVettingList
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandVettingList
        """
        if self._brand_vettings is None:
            self._brand_vettings = BrandVettingList(
                self._version,
                self._solution["sid"],
            )
        return self._brand_vettings

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Messaging.V1.BrandRegistrationContext {}>".format(context)


class BrandRegistrationPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of BrandRegistrationInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        return BrandRegistrationInstance(self._version, payload)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Messaging.V1.BrandRegistrationPage>"


class BrandRegistrationList(ListResource):
    def __init__(self, version: Version):
        """
        Initialize the BrandRegistrationList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.messaging.v1.brand_registration.BrandRegistrationList
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationList
        """
        super().__init__(version)

        self._uri = "/a2p/BrandRegistrations"

    def create(
        self,
        customer_profile_bundle_sid,
        a2p_profile_bundle_sid,
        brand_type=values.unset,
        mock=values.unset,
        skip_automatic_sec_vet=values.unset,
    ):
        """
        Create the BrandRegistrationInstance

        :param str customer_profile_bundle_sid: Customer Profile Bundle Sid.
        :param str a2p_profile_bundle_sid: A2P Messaging Profile Bundle Sid.
        :param str brand_type: Type of brand being created. One of: \\\"STANDARD\\\", \\\"STARTER\\\". STARTER is for low volume, starter use cases. STANDARD is for all other use cases.
        :param bool mock: A boolean that specifies whether brand should be a mock or not. If true, brand will be registered as a mock brand. Defaults to false if no value is provided.
        :param bool skip_automatic_sec_vet: A flag to disable automatic secondary vetting for brands which it would otherwise be done.

        :returns: The created BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        data = values.of(
            {
                "CustomerProfileBundleSid": customer_profile_bundle_sid,
                "A2PProfileBundleSid": a2p_profile_bundle_sid,
                "BrandType": brand_type,
                "Mock": mock,
                "SkipAutomaticSecVet": skip_automatic_sec_vet,
            }
        )

        payload = self._version.create(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return BrandRegistrationInstance(self._version, payload)

    async def create_async(
        self,
        customer_profile_bundle_sid,
        a2p_profile_bundle_sid,
        brand_type=values.unset,
        mock=values.unset,
        skip_automatic_sec_vet=values.unset,
    ):
        """
        Asynchronously create the BrandRegistrationInstance

        :param str customer_profile_bundle_sid: Customer Profile Bundle Sid.
        :param str a2p_profile_bundle_sid: A2P Messaging Profile Bundle Sid.
        :param str brand_type: Type of brand being created. One of: \\\"STANDARD\\\", \\\"STARTER\\\". STARTER is for low volume, starter use cases. STANDARD is for all other use cases.
        :param bool mock: A boolean that specifies whether brand should be a mock or not. If true, brand will be registered as a mock brand. Defaults to false if no value is provided.
        :param bool skip_automatic_sec_vet: A flag to disable automatic secondary vetting for brands which it would otherwise be done.

        :returns: The created BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance
        """
        data = values.of(
            {
                "CustomerProfileBundleSid": customer_profile_bundle_sid,
                "A2PProfileBundleSid": a2p_profile_bundle_sid,
                "BrandType": brand_type,
                "Mock": mock,
                "SkipAutomaticSecVet": skip_automatic_sec_vet,
            }
        )

        payload = await self._version.create_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return BrandRegistrationInstance(self._version, payload)

    def stream(self, limit=None, page_size=None):
        """
        Streams BrandRegistrationInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(self, limit=None, page_size=None):
        """
        Asynchronously streams BrandRegistrationInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(page_size=limits["page_size"])

        return await self._version.stream_async(page, limits["limit"])

    def list(self, limit=None, page_size=None):
        """
        Lists BrandRegistrationInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance]
        """
        return list(
            self.stream(
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(self, limit=None, page_size=None):
        """
        Asynchronously lists BrandRegistrationInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.messaging.v1.brand_registration.BrandRegistrationInstance]
        """
        return list(
            await self.stream_async(
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Retrieve a single page of BrandRegistrationInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return BrandRegistrationPage(self._version, response)

    async def page_async(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Asynchronously retrieve a single page of BrandRegistrationInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return BrandRegistrationPage(self._version, response)

    def get_page(self, target_url):
        """
        Retrieve a specific page of BrandRegistrationInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return BrandRegistrationPage(self._version, response)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of BrandRegistrationInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of BrandRegistrationInstance
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return BrandRegistrationPage(self._version, response)

    def get(self, sid):
        """
        Constructs a BrandRegistrationContext

        :param sid: The SID of the Brand Registration resource to update.

        :returns: twilio.rest.messaging.v1.brand_registration.BrandRegistrationContext
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationContext
        """
        return BrandRegistrationContext(self._version, sid=sid)

    def __call__(self, sid):
        """
        Constructs a BrandRegistrationContext

        :param sid: The SID of the Brand Registration resource to update.

        :returns: twilio.rest.messaging.v1.brand_registration.BrandRegistrationContext
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationContext
        """
        return BrandRegistrationContext(self._version, sid=sid)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Messaging.V1.BrandRegistrationList>"
